@@ -1,11 +1,12 @@
 import copy
+import datetime
 import re
 
 from wikibaseintegrator.wbi_config import config
 from wikibaseintegrator.wbi_jsonparser import JsonParser
 
 
-class BaseDataType(object):
+class BaseDataType:
     """
     The base class for all Wikibase data types, they inherit from it
     """
@@ -64,8 +65,16 @@ class BaseDataType(object):
         if not self.references:
             self.references = []
         else:
+            if isinstance(self.references, BaseDataType):
+                self.references = [[self.references]]
+
             for ref_list in self.references:
+                if isinstance(ref_list, BaseDataType):
+                    ref_list = [ref_list]
                 for reference in ref_list:
+                    if not isinstance(reference, BaseDataType):
+                        raise ValueError('A reference must be an instance of class BaseDataType.')
+
                     if reference.is_reference is False:
                         raise ValueError('A reference can\'t be declared as is_reference=False')
                     elif reference.is_reference is None:
@@ -74,7 +83,12 @@ class BaseDataType(object):
         if not self.qualifiers:
             self.qualifiers = []
         else:
+            if isinstance(self.qualifiers, BaseDataType):
+                self.qualifiers = [self.qualifiers]
+
             for qualifier in self.qualifiers:
+                if not isinstance(qualifier, BaseDataType):
+                    raise ValueError('A qualifier must be an instance of class BaseDataType.')
                 if qualifier.is_qualifier is False:
                     raise ValueError('A qualifier can\'t be declared as is_qualifier=False')
                 elif qualifier.is_qualifier is None:
@@ -87,7 +101,7 @@ class BaseDataType(object):
             matches = pattern.match(prop_nr)
 
             if not matches:
-                raise ValueError('Invalid prop_nr, format must be "P[0-9]+"')
+                raise ValueError(f'Invalid prop_nr, format must be "P[0-9]+", got {prop_nr}')
             else:
                 self.prop_nr = 'P' + str(matches.group(1))
 
@@ -103,10 +117,10 @@ class BaseDataType(object):
         }
 
         if self.snak_type not in ['value', 'novalue', 'somevalue']:
-            raise ValueError('{} is not a valid snak type'.format(self.snak_type))
+            raise ValueError(f'{self.snak_type} is not a valid snak type')
 
         if self.if_exists not in ['REPLACE', 'APPEND', 'FORCE_APPEND', 'KEEP']:
-            raise ValueError('{} is not a valid if_exists value'.format(self.if_exists))
+            raise ValueError(f'{self.if_exists} is not a valid if_exists value')
 
         if self.value is None and self.snak_type == 'value':
             raise ValueError('Parameter \'value\' can\'t be \'None\' if \'snak_type\' is \'value\'')
@@ -214,7 +228,7 @@ class BaseDataType(object):
         valid_ranks = ['normal', 'deprecated', 'preferred']
 
         if rank not in valid_ranks:
-            raise ValueError("{} not a valid rank".format(rank))
+            raise ValueError(f"{rank} not a valid rank")
 
         self.rank = rank
 
@@ -350,7 +364,7 @@ class BaseDataType(object):
         return "<{klass} @{id:x} {attrs}>".format(
             klass=self.__class__.__name__,
             id=id(self) & 0xFFFFFF,
-            attrs=" ".join("{}={!r}".format(k, v) for k, v in self.__dict__.items()),
+            attrs=" ".join(f"{k}={v!r}" for k, v in self.__dict__.items()),
         )
 
 
@@ -383,7 +397,64 @@ class CommonsMedia(BaseDataType):
 
         self.value = None
 
-        super(CommonsMedia, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
+
+        self.set_value(value)
+
+    def set_value(self, value):
+        assert isinstance(value, str) or value is None, f"Expected str, found {type(value)} ({value})"
+        self.value = value
+
+        self.json_representation['datavalue'] = {
+            'value': self.value,
+            'type': 'string'
+        }
+
+        super().set_value(value=self.value)
+
+    @classmethod
+    @JsonParser
+    def from_json(cls, jsn):
+        if jsn['snaktype'] == 'novalue' or jsn['snaktype'] == 'somevalue':
+            return cls(value=None, prop_nr=jsn['property'], snak_type=jsn['snaktype'])
+        return cls(value=jsn['datavalue']['value'], prop_nr=jsn['property'])
+
+
+class EDTF(BaseDataType):
+    """
+    Implements the data type for Extended Date/Time Format (EDTF) extension.
+    More info: https://github.com/ProfessionalWiki/WikibaseEdtf
+    """
+    DTYPE = 'edtf'
+    sparql_query = '''
+        SELECT * WHERE {{
+          ?item_id <{wb_url}/prop/{pid}> ?s .
+          ?s <{wb_url}/prop/statement/{pid}> '{value}'^^xsd:edtf .
+        }}
+    '''
+
+    def __init__(self, value, prop_nr, **kwargs):
+        """
+        Constructor, calls the superclass BaseDataType
+        :param value: Value using the Extended Date/Time Format (EDTF)
+        :type value: str with a 'P' prefix, followed by several digits or only the digits without the 'P' prefix
+        :param prop_nr: The property number for this claim
+        :type prop_nr: str with a 'P' prefix followed by digits
+        :param is_reference: Whether this snak is a reference
+        :type is_reference: boolean
+        :param is_qualifier: Whether this snak is a qualifier
+        :type is_qualifier: boolean
+        :param snak_type: The snak type, either 'value', 'somevalue' or 'novalue'
+        :type snak_type: str
+        :param references: List with reference objects
+        :type references: A data type with subclass of BaseDataType
+        :param qualifiers: List with qualifier objects
+        :type qualifiers: A data type with subclass of BaseDataType
+        :param rank: rank of a snak with value 'preferred', 'normal' or 'deprecated'
+        :type rank: str
+        """
+
+        super(EDTF, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
@@ -396,7 +467,7 @@ class CommonsMedia(BaseDataType):
             'type': 'string'
         }
 
-        super(CommonsMedia, self).set_value(value=self.value)
+        super(EDTF, self).set_value(value=self.value)
 
     @classmethod
     @JsonParser
@@ -433,12 +504,12 @@ class ExternalID(BaseDataType):
         :type rank: str
         """
 
-        super(ExternalID, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
-        assert isinstance(value, str) or value is None, "Expected str, found {} ({})".format(type(value), value)
+        assert isinstance(value, str) or value is None, f"Expected str, found {type(value)} ({value})"
         self.value = value
 
         self.json_representation['datavalue'] = {
@@ -446,7 +517,7 @@ class ExternalID(BaseDataType):
             'type': 'string'
         }
 
-        super(ExternalID, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     @classmethod
     @JsonParser
@@ -489,12 +560,12 @@ class Form(BaseDataType):
         :type rank: str
         """
 
-        super(Form, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
-        assert isinstance(value, str) or value is None, "Expected str, found {} ({})".format(type(value), value)
+        assert isinstance(value, str) or value is None, f"Expected str, found {type(value)} ({value})"
         if value is None:
             self.value = value
         else:
@@ -502,7 +573,7 @@ class Form(BaseDataType):
             matches = pattern.match(value)
 
             if not matches:
-                raise ValueError("Invalid form ID ({}), format must be 'L[0-9]+-F[0-9]+'".format(value))
+                raise ValueError(f"Invalid form ID ({value}), format must be 'L[0-9]+-F[0-9]+'")
 
             self.value = value
 
@@ -514,7 +585,7 @@ class Form(BaseDataType):
             'type': 'wikibase-entityid'
         }
 
-        super(Form, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     @classmethod
     @JsonParser
@@ -557,12 +628,12 @@ class GeoShape(BaseDataType):
         :type rank: str
         """
 
-        super(GeoShape, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
-        assert isinstance(value, str) or value is None, "Expected str, found {} ({})".format(type(value), value)
+        assert isinstance(value, str) or value is None, f"Expected str, found {type(value)} ({value})"
         if value is None:
             self.value = value
         else:
@@ -578,7 +649,7 @@ class GeoShape(BaseDataType):
             'type': 'string'
         }
 
-        super(GeoShape, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     @classmethod
     @JsonParser
@@ -638,7 +709,7 @@ class GlobeCoordinate(BaseDataType):
 
         value = (latitude, longitude, precision, globe)
 
-        super(GlobeCoordinate, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
@@ -658,10 +729,10 @@ class GlobeCoordinate(BaseDataType):
         }
 
         self.value = (self.latitude, self.longitude, self.precision, self.globe)
-        super(GlobeCoordinate, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     def get_sparql_value(self):
-        return 'Point(' + str(self.latitude) + ', ' + str(self.longitude) + ')'
+        return 'Point(' + str(self.longitude) + ' ' + str(self.latitude) + ')'
 
     @classmethod
     @JsonParser
@@ -708,12 +779,12 @@ class ItemID(BaseDataType):
         :type rank: str
         """
 
-        super(ItemID, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
-        assert isinstance(value, (str, int)) or value is None, 'Expected str or int, found {} ({})'.format(type(value), value)
+        assert isinstance(value, (str, int)) or value is None, f'Expected str or int, found {type(value)} ({value})'
         if value is None:
             self.value = value
         elif isinstance(value, int):
@@ -723,7 +794,7 @@ class ItemID(BaseDataType):
             matches = pattern.match(value)
 
             if not matches:
-                raise ValueError("Invalid item ID ({}), format must be 'Q[0-9]+'".format(value))
+                raise ValueError(f"Invalid item ID ({value}), format must be 'Q[0-9]+'")
             else:
                 self.value = int(matches.group(1))
 
@@ -731,12 +802,12 @@ class ItemID(BaseDataType):
             'value': {
                 'entity-type': 'item',
                 'numeric-id': self.value,
-                'id': 'Q{}'.format(self.value)
+                'id': f'Q{self.value}'
             },
             'type': 'wikibase-entityid'
         }
 
-        super(ItemID, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     @classmethod
     @JsonParser
@@ -779,12 +850,12 @@ class Lexeme(BaseDataType):
         :type rank: str
         """
 
-        super(Lexeme, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
-        assert isinstance(value, (str, int)) or value is None, "Expected str or int, found {} ({})".format(type(value), value)
+        assert isinstance(value, (str, int)) or value is None, f"Expected str or int, found {type(value)} ({value})"
         if value is None:
             self.value = value
         elif isinstance(value, int):
@@ -794,7 +865,7 @@ class Lexeme(BaseDataType):
             matches = pattern.match(value)
 
             if not matches:
-                raise ValueError("Invalid lexeme ID ({}), format must be 'L[0-9]+'".format(value))
+                raise ValueError(f"Invalid lexeme ID ({value}), format must be 'L[0-9]+'")
             else:
                 self.value = int(matches.group(1))
 
@@ -802,12 +873,12 @@ class Lexeme(BaseDataType):
             'value': {
                 'entity-type': 'lexeme',
                 'numeric-id': self.value,
-                'id': 'L{}'.format(self.value)
+                'id': f'L{self.value}'
             },
             'type': 'wikibase-entityid'
         }
 
-        super(Lexeme, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     @classmethod
     @JsonParser
@@ -844,12 +915,12 @@ class Math(BaseDataType):
         :type rank: str
         """
 
-        super(Math, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
-        assert isinstance(value, str) or value is None, "Expected str, found {} ({})".format(type(value), value)
+        assert isinstance(value, str) or value is None, f"Expected str, found {type(value)} ({value})"
         self.value = value
 
         self.json_representation['datavalue'] = {
@@ -857,7 +928,7 @@ class Math(BaseDataType):
             'type': 'string'
         }
 
-        super(Math, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     @classmethod
     @JsonParser
@@ -907,17 +978,17 @@ class MonolingualText(BaseDataType):
 
         value = (text, self.language)
 
-        super(MonolingualText, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
         self.text, self.language = value
         if self.text is not None:
-            assert isinstance(self.text, str) or self.text is None, "Expected str, found {} ({})".format(type(self.text), self.text)
+            assert isinstance(self.text, str) or self.text is None, f"Expected str, found {type(self.text)} ({self.text})"
         elif self.snak_type == 'value':
             raise ValueError("Parameter 'text' can't be 'None' if 'snak_type' is 'value'")
-        assert isinstance(self.language, str), "Expected str, found {} ({})".format(type(self.language), self.language)
+        assert isinstance(self.language, str), f"Expected str, found {type(self.language)} ({self.language})"
 
         self.json_representation['datavalue'] = {
             'value': {
@@ -928,7 +999,7 @@ class MonolingualText(BaseDataType):
         }
 
         self.value = (self.text, self.language)
-        super(MonolingualText, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     def get_sparql_value(self):
         return '"' + self.text.replace('"', r'\"') + '"@' + self.language
@@ -970,12 +1041,12 @@ class MusicalNotation(BaseDataType):
         :type rank: str
         """
 
-        super(MusicalNotation, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
-        assert isinstance(value, str) or value is None, "Expected str, found {} ({})".format(type(value), value)
+        assert isinstance(value, str) or value is None, f"Expected str, found {type(value)} ({value})"
         self.value = value
 
         self.json_representation['datavalue'] = {
@@ -983,7 +1054,7 @@ class MusicalNotation(BaseDataType):
             'type': 'string'
         }
 
-        super(MusicalNotation, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     @classmethod
     @JsonParser
@@ -1026,12 +1097,12 @@ class Property(BaseDataType):
         :type rank: str
         """
 
-        super(Property, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
-        assert isinstance(value, (str, int)) or value is None, "Expected str or int, found {} ({})".format(type(value), value)
+        assert isinstance(value, (str, int)) or value is None, f"Expected str or int, found {type(value)} ({value})"
         if value is None:
             self.value = value
         elif isinstance(value, int):
@@ -1041,7 +1112,7 @@ class Property(BaseDataType):
             matches = pattern.match(value)
 
             if not matches:
-                raise ValueError("Invalid property ID ({}), format must be 'P[0-9]+'".format(value))
+                raise ValueError(f"Invalid property ID ({value}), format must be 'P[0-9]+'")
             else:
                 self.value = int(matches.group(1))
 
@@ -1049,12 +1120,12 @@ class Property(BaseDataType):
             'value': {
                 'entity-type': 'property',
                 'numeric-id': self.value,
-                'id': 'P{}'.format(self.value)
+                'id': f'P{self.value}'
             },
             'type': 'wikibase-entityid'
         }
 
-        super(Property, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     @classmethod
     @JsonParser
@@ -1115,7 +1186,7 @@ class Quantity(BaseDataType):
 
         value = (quantity, unit, upper_bound, lower_bound)
 
-        super(Quantity, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
@@ -1165,7 +1236,7 @@ class Quantity(BaseDataType):
             del self.json_representation['datavalue']['value']['lowerBound']
 
         self.value = (self.quantity, self.unit, self.upper_bound, self.lower_bound)
-        super(Quantity, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     def get_sparql_value(self):
         return self.quantity
@@ -1190,7 +1261,7 @@ class Quantity(BaseDataType):
 
         # Adding prefix + for positive number and 0
         if not str(amount).startswith('+') and float(amount) >= 0:
-            amount = str('+{}'.format(amount))
+            amount = str(f'+{amount}')
 
         # return as string
         return str(amount)
@@ -1229,12 +1300,12 @@ class Sense(BaseDataType):
         :type rank: str
         """
 
-        super(Sense, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
-        assert isinstance(value, str) or value is None, "Expected str, found {} ({})".format(type(value), value)
+        assert isinstance(value, str) or value is None, f"Expected str, found {type(value)} ({value})"
         if value is None:
             self.value = value
         else:
@@ -1242,7 +1313,7 @@ class Sense(BaseDataType):
             matches = pattern.match(value)
 
             if not matches:
-                raise ValueError("Invalid sense ID ({}), format must be 'L[0-9]+-S[0-9]+'".format(value))
+                raise ValueError(f"Invalid sense ID ({value}), format must be 'L[0-9]+-S[0-9]+'")
 
             self.value = value
 
@@ -1254,7 +1325,7 @@ class Sense(BaseDataType):
             'type': 'wikibase-entityid'
         }
 
-        super(Sense, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     @classmethod
     @JsonParser
@@ -1292,12 +1363,12 @@ class String(BaseDataType):
         :type rank: str
         """
 
-        super(String, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
-        assert isinstance(value, str) or value is None, "Expected str, found {} ({})".format(type(value), value)
+        assert isinstance(value, str) or value is None, f"Expected str, found {type(value)} ({value})"
         self.value = value
 
         self.json_representation['datavalue'] = {
@@ -1305,7 +1376,7 @@ class String(BaseDataType):
             'type': 'string'
         }
 
-        super(String, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     @classmethod
     @JsonParser
@@ -1342,12 +1413,12 @@ class TabularData(BaseDataType):
         :type rank: str
         """
 
-        super(TabularData, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
-        assert isinstance(value, str) or value is None, "Expected str, found {} ({})".format(type(value), value)
+        assert isinstance(value, str) or value is None, f"Expected str, found {type(value)} ({value})"
         if value is None:
             self.value = value
         else:
@@ -1363,7 +1434,7 @@ class TabularData(BaseDataType):
             'type': 'string'
         }
 
-        super(TabularData, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     @classmethod
     @JsonParser
@@ -1389,7 +1460,7 @@ class Time(BaseDataType):
         """
         Constructor, calls the superclass BaseDataType
         :param time: Explicit value for point in time, represented as a timestamp resembling ISO 8601
-        :type time: str in the format '+%Y-%m-%dT%H:%M:%SZ', e.g. '+2001-12-31T12:01:13Z'
+        :type time: str in the format '+%Y-%m-%dT%H:%M:%SZ', e.g. '+2001-12-31T12:01:13Z' or 'now'
         :param prop_nr: The property number for this claim
         :type prop_nr: str with a 'P' prefix followed by digits
         :param before: explicit integer value for how many units after the given time it could be.
@@ -1434,15 +1505,18 @@ class Time(BaseDataType):
 
         value = (time, before, after, precision, timezone, calendarmodel)
 
-        super(Time, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
         self.time, self.before, self.after, self.precision, self.timezone, self.calendarmodel = value
-        assert isinstance(self.time, str) or self.time is None, "Expected str, found {} ({})".format(type(self.time), self.time)
+        assert isinstance(self.time, str) or self.time is None, f"Expected str, found {type(self.time)} ({self.time})"
 
         if self.time is not None:
+            if self.time == "now":
+                self.time = datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ")
+
             if not (self.time.startswith("+") or self.time.startswith("-")):
                 self.time = "+" + self.time
             pattern = re.compile(r'^[+-][0-9]*-(?:1[0-2]|0[0-9])-(?:3[01]|0[0-9]|[12][0-9])T(?:2[0-3]|[01][0-9]):[0-5][0-9]:[0-5][0-9]Z$')
@@ -1468,7 +1542,7 @@ class Time(BaseDataType):
         }
 
         self.value = (self.time, self.before, self.after, self.precision, self.timezone, self.calendarmodel)
-        super(Time, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     def get_sparql_value(self):
         return self.time
@@ -1517,12 +1591,12 @@ class Url(BaseDataType):
         :type rank: str
         """
 
-        super(Url, self).__init__(value=value, prop_nr=prop_nr, **kwargs)
+        super().__init__(value=value, prop_nr=prop_nr, **kwargs)
 
         self.set_value(value)
 
     def set_value(self, value):
-        assert isinstance(value, str) or value is None, "Expected str, found {} ({})".format(type(value), value)
+        assert isinstance(value, str) or value is None, f"Expected str, found {type(value)} ({value})"
         if value is None:
             self.value = value
         else:
@@ -1530,7 +1604,7 @@ class Url(BaseDataType):
             matches = pattern.match(value)
 
             if not matches:
-                raise ValueError("Invalid URL {}".format(value))
+                raise ValueError(f"Invalid URL {value}")
             self.value = value
 
         self.json_representation['datavalue'] = {
@@ -1538,7 +1612,7 @@ class Url(BaseDataType):
             'type': 'string'
         }
 
-        super(Url, self).set_value(value=self.value)
+        super().set_value(value=self.value)
 
     @classmethod
     @JsonParser
